﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Methods
{
    class Methods
    {
        // Parameters default to pass-by-value, so changes to
        // built-in data type values do not persist
        public static void foo1(int i)
        {
            System.Console.WriteLine("FOO1 i = {0}", i);
            i++;
            System.Console.WriteLine("FOO1 i = {0}", i);
        }

        // Methods can declare arguments as ref to pass-by-reference
        public static void foo2(ref int i)
        {
            System.Console.WriteLine("FOO1 2 = {0}", i);
            i++;
            System.Console.WriteLine("FOO1 2 = {0}", i);
        }

        // We can change the values stored in arrays or mutaute other reference types
        public static void foo3(int[] nums)
        {
            for (int i = 0; i < nums.Length; i++)
            {
                nums[i] = 0;
            }
        }

        // Example of using the out keyword to allow output via uninitialized parameters
        public static void foo4(out int num1, out int num2)
        {
            num1 = 100;
            num2 = 200;
        }

        static void Main(string[] args)
        {
            // Normal pass-by-value example
            int i = 10;            
            System.Console.WriteLine("MAIN i = {0}", i);
            foo1(i);
            System.Console.WriteLine("MAIN i = {0}", i);

            // To pass i by reference, we must also use ref keyword
            foo2(ref i);
            System.Console.WriteLine("MAIN i = {0}", i);

            // Methods are allowed to change the contents of reference types, e.g. array elements
            int[] point = { 1, 2 };
            System.Console.WriteLine("MAIN point = {0} {1}", point[0], point[1]);
            foo3(point);
            System.Console.WriteLine("MAIN point = {0} {1}", point[0], point[1]);

            // Sometimes we want to pass-by-reference for purposes of getting multiple result values.
            // out keyword denotes this and doesn't require initialization of output parameters.
            int a;
            int b;
            foo4(out a, out b);
            System.Console.WriteLine("MAIN a = {0}, b = {1}", a, b);
        }
    }
}
