﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OOP
{
    class Person
    {
        private string first = "";
        private string last = "";

        // Access modifiers
        public              int a;  // Accessed from any assembly
        private             int b;  // Accessed from same class or struct
        protected           int c;  // Accessed from same class or struct, or in a class dervied from this class
        internal            int d;  // Accessed from same assembly, but not from another assembly
        protected internal  int e;  // Accessed from same assembly or derived classes in another assembly

        public Person(string first, string last)
        {
            this.first = first;
            this.last = last;
        }

        public void foo1(int a)
        {
            System.Console.WriteLine("FOO1 PERSON a = {0}", a);
        }

        // virtual keyword marks us as a method we let children override
        public virtual void foo2(int a, int b)
        {
            System.Console.WriteLine("FOO2 PERSON a = {0}, b = {1}", a, b);
        }

        public virtual void foo3(int a, int b, int c)
        {
            System.Console.WriteLine("FOO3 PERSON a = {0}, b = {1}, c = {2}", a, b, c);
        }
    }

    class Student : Person
    {
        private double gpa = 0.0;

        public Student(string first, string last, double gpa) : base(first, last)
        {
            this.gpa = gpa;
        }

        // Example of using this this keyword to use a constructor of our same class
        public Student(string first, string last) : this(first, last, 0.0)
        {
        }

        // We are hiding our base class implement of foo(int), we get a warning unless we use new keyword.
        public new void foo1(int a)
        {
            System.Console.WriteLine("FOO1 STUDENT a = {0}", a);
        }

        public override void foo2(int a, int b)
        {
            base.foo2(a, b);
            System.Console.WriteLine("FOO2 STUDENT a = {0}, b = {1}", a, b);
        }

        // We can prevent children for overriding using sealed keyword
        public sealed override void foo3(int a, int b, int c)
        {
            System.Console.WriteLine("FOO3 STUDENT a = {0}, b = {1}, c = {2}", a, b, c);
        }
    }

    sealed class Worker : Student
    {
        private double wage = 0.0;

        public Worker(string first, string last, double gpa, double wage) : base(first, last, gpa)
        {
            this.wage = wage;
        }

        // This will not compile since our parent method is declared as sealed
        //public override void foo3(int a, int b, int c)
        //{
        //    System.Console.WriteLine("FOO3 Worker a = {0}, b = {1}, c = {2}", a, b, c);
        //}

    }

    // This will not compile since parent class is declared sealed
    // class Tutor : Worker
    // {
    // }

    class OOP
    {
        static void Main(string[] args)
        {
            Student student = new Student("John", "Doe", 3.3);
            student.foo1(1);
            student.foo2(2, 3);
            System.Console.WriteLine();

            Person p = (Person)student;
            p.foo1(1);
            p.foo2(2, 3);
        }
    }
}

