﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Types
{
    class Types
    {
        static void Main(string[] args)
        {
            // All the built-in data types in C#
            byte    b;      // 0..255
            sbyte   sb;     // -128..127
            short   s;      // -32,768..32,767
            ushort  us;     // 0..65,535
            int     i;      // -2,147,483,648..2,147,483,647
            uint    ui;     // 0..4,294,967,295
            long    l;      // -9,223,372,036,854,775,808..9,223,372,036,854,775,807
            ulong   ul;     // 0..18,446,744,073,709,551,615
            float   f;      // -3.402823e38..3.402823e38 (7 sig figs)
            double  d;      // -1.79769313486232e308..1.79769313486232e308 (15/16 sig figs)
            decimal dec;    // -79228162514264337593543950335..79228162514264337593543950335 (28 sig figs)
            char    c;      // A Unicode character
            string  str;    // A string of Unicode characters
            bool    flag;   // true or false
            object  o;      // An object

            // All .NET datatypes can be treated as if they were classes, including literals
            i = 103;
            string test = i.ToString();
            string test2 = 103.ToString();
            System.Console.WriteLine("test = {0}, test2 = {1}", test, test2);

            // const is used to declare something that can't be changed
            const double SPEED_OF_LIGHT = 299792458.0;                        
            // SPEED_OF_LIGHT = 0.0;  // compile error

            // By default, you can "roll around" if you overflow a data type
            b = 255;
            System.Console.WriteLine("b = {0}", b);
            b++;
            System.Console.WriteLine("b = {0}", b);

            // You can request strict overflow checking
            checked
            {
                b = 255;
                System.Console.WriteLine("b = {0}", b);
                // b++; // causes runtime exception
                System.Console.WriteLine("b = {0}", b);
            }

            // With a const variable, you must assign it a value
            // const double BLAH; // compile error

            Types types = new Types();
            types.foo();

            // Structs are supported, they are value types stored on stack.
            // You can create with new if you want.
            Point p = new Point();
            p.x = 2.0;
            p.y = 3.0;
            System.Console.WriteLine("p = {0}", p);

            // Or you can use declare and start using
            Point p2;
            p2.x = 2.0;
            p2.y = 3.0;
            System.Console.WriteLine("p = {0}", p);

        }

        /////////////////////////////////////////////////////////////////////////////////
        private readonly int ID;
        private readonly int SECRET = 42;

        public Types()
        {
            ID = 123;
            SECRET = 5;
            System.Console.WriteLine("ID = {0}, SECRET = {1}", ID, SECRET);
        }

        public void foo()
        {
            // ID = 123;   // copmile error
            // SECRET = 5; // copmile error
        }

        /////////////////////////////////////////////////////////////////////////////////
        struct Point
        {
            public double x;
            public double y;

            // Structs can have constructors (must have arguments however) 
            public Point(double x, double y)
            {
                this.x = x;
                this.y = y;
            }

            // Structs can also override System.object methods
            public override string ToString()
            {
                return "(" + x + "," + y + ")";
            }

        }

    }
}
